/**********************************************************************
 * options.c                                                   May 1999
 * Horms                                            horms@vergenet.net.
 * Verge Systems International                    http://vergenet.net./
 *
 * Read in command line options
 * Code based on man getopt(3)
 **********************************************************************/

#include "options.h"


/**********************************************************************
 * perdition_options
 * Read in command line options
 **********************************************************************/
perdition_options_t perdition_options(int argc, char **argv){
  int c;
  perdition_options_t options;

  options.mapname=DEFAULT_MAPNAME;
  options.inetd_mode=0;
  options.port=DEFAULT_PORT;
  options.username=DEFAULT_USERNAME;
  options.group=DEFAULT_GROUP;

  while (1){
    int option_index = 0;
    static struct option long_options[] =
    {
      {"map_name", 1, 0, 0},
      {"inetd_mode", 0, 0, 0},
      {"port", 1, 0, 0},
      {"username", 1, 0, 0},
      {"group", 1, 0, 0},
      {"help", 0, 0, 0},
      {"jain", 2, 0, 0},
      {"jane", 2, 0, 0},
      {"jayne", 2, 0, 0},
      {0, 0, 0, 0}
    };

    c = getopt_long (argc, argv, "m:ip:u:g:hj", long_options, &option_index);
    if (c == -1){
      return(options);
    }

    switch (c){
      case 0:
        if(strcmp(long_options[option_index].name,"help")==0){
          usage(0);
          break;
        }
        if(str_is_jain(long_options[option_index].name)){
          printf(
            "%s, Oath\n", 
            long_options[option_index].name
          );
          break;
        }
        if(strcmp(long_options[option_index].name,"mapname")==0){
          options.mapname=optarg;
          break;
        }
        if(strcmp(long_options[option_index].name,"inetd_mode")==0){
          options.inetd_mode=1;
          break;
        }
        if(strcmp(long_options[option_index].name,"port")==0){
          options.port=optarg;
          break;
        }
        if(strcmp(long_options[option_index].name,"username")==0){
          options.username=optarg;
          break;
        }
        if(strcmp(long_options[option_index].name,"group")==0){
          options.group=optarg;
          break;
        }
        break;
      case 'h':
        usage(0);
        break;
      case 'j':
        printf("Jain, Oath\n");
        break;
      case 'm':
        options.mapname=optarg;
        break;
      case 'i':
        options.inetd_mode=1;
        break;
      case 'p':
        options.port=optarg;
        break;
      case 'u':
        options.username=optarg;
        break;
      case 'g':
        options.group=optarg;
        break;
      case '?':
        usage(-1);
        break;
      default:
        printf ("?? getopt returned character code 0%o ??\n", c);
      }
  }

  if (optind<argc) {
    usage(-1);
  }

  return(options);
}


/**********************************************************************
 * log_options
 * Log options to syslog
 **********************************************************************/

void log_options(perdition_options_t options){
  syslog(
    LOG_INFO, 
    "mapname=%s, inetd_mode=%d, port=%s, username=%s, group=%s\n",
    options.mapname,
    options.inetd_mode,
    options.port,
    options.username,
    options.group
  );
}


/**********************************************************************
 * usage
 * Display usage unfromtaion
 * Printed to stdout if exit_status=0, stderr otherwise
 **********************************************************************/

void usage(int exit_status){
  FILE *stream;

  if(exit_status!=0){
     stream=stderr;
  }
  else{
     stream=stdout;
  }

    
  fprintf(
    stream, 
    "Usage: perdition [options]\n"
    "  options:\n"
    "     -h|--help:       Display this message\n"
    "     -m|--mapname:    gdbm map for users to hosts and port\n"
    "                        (default %s)\n"
    "     -i|--inetd_mode: Run in inetdmode\n"
    "     -p|--port:       Port to listen on (default %s)\n"
    "     -u|--username:   Username to run as (default %s)\n"
    "     -g|--group:      Group to run as (default %s)\n",
    DEFAULT_MAPNAME,
    DEFAULT_PORT,
    DEFAULT_USERNAME,
    DEFAULT_GROUP
  );

  exit(exit_status);
}
