/**********************************************************************
 * perdition.c                                               March 1999
 * Horms                                            horms@vergenet.net.
 * Verge Systems International                    http://vergenet.net./
 *
 * pop3 forwarder
 **********************************************************************/

#include "perdition.h"

/**********************************************************************
 * Muriel the main funtion
 **********************************************************************/

int main (int argc, char **argv){
  struct passwd *pw;
  int client_in;
  int client_out;
  int server;
#ifdef RETRY
  int retry;
#endif
  unsigned char *buffer;
  perdition_options_t options;
  server_port_t *server_port;
  char *servername;
  char *port;

  /*Parse options*/
  options=perdition_options(argc, argv);

  /*Open syslog*/
  openlog(LOG_IDENT, LOG_OPTION, LOG_FACILITY);

  /*Set signal handlers*/
  /*signal(SIGHUP,    (void(*)(int))exit_cleanly);*/
  signal(SIGINT,    (void(*)(int))exit_cleanly);
  signal(SIGQUIT,   (void(*)(int))exit_cleanly);
  signal(SIGILL,    (void(*)(int))exit_cleanly);
  signal(SIGTRAP,   (void(*)(int))exit_cleanly);
  signal(SIGIOT,    (void(*)(int))exit_cleanly);
  signal(SIGBUS,    (void(*)(int))exit_cleanly);
  signal(SIGFPE,    (void(*)(int))exit_cleanly);
  signal(SIGUSR1,   (void(*)(int))exit_cleanly);
  signal(SIGSEGV,   (void(*)(int))exit_cleanly);
  signal(SIGUSR2,   (void(*)(int))exit_cleanly);
  signal(SIGPIPE,   (void(*)(int))exit_cleanly);
  signal(SIGALRM,   (void(*)(int))exit_cleanly);
  signal(SIGTERM,   (void(*)(int))exit_cleanly);
  signal(SIGURG,    (void(*)(int))exit_cleanly);
  signal(SIGXCPU,   (void(*)(int))exit_cleanly);
  signal(SIGXFSZ,   (void(*)(int))exit_cleanly);
  signal(SIGVTALRM, (void(*)(int))exit_cleanly);
  signal(SIGPROF,   (void(*)(int))exit_cleanly);
  signal(SIGWINCH,  (void(*)(int))exit_cleanly);
  signal(SIGIO,     (void(*)(int))exit_cleanly);

  /*Close file descriptors and detactch process from shell as necessary*/
  if(options.inetd_mode){
    daemon_inetd_process();
  }
  else{
    daemon_process();
  }

  /*Tell the world we are here */
  syslog( LOG_INFO, "Started");
  log_options(options);

  /*Open incomins socket at required*/
  if(options.inetd_mode){
    client_in=0;
    client_out=1;
  }
  else{
    if((client_in=tcp_socket_server_connect(options.port))<0){
      syslog(LOG_WARNING, "main: tcp_socket_server_connect");
      return(-1);
    }
    client_out=client_in;
  }

#ifdef AUTHENTICATE_IN
  if(geteuid()){
    syslog(
      LOG_INFO, 
      "Warning: not invoked as root, local authentication may fail"
    );
  }
#else
  /*Become someone else*/
  /*NB: We do this later if we are going to authenticate locally*/
  if(geteuid()){
    syslog(LOG_INFO, "Warning: not invoked as root, not setting uid and gid");
  }
  else if(daemon_setid(options.username, options.group)){
    syslog(LOG_WARNING, "Error setting group and userid, Exiting");
    exit(-1);
  }
#endif

  /*Speak to our client*/
  if(write_strings(client_out, 4, OK, " ", GREETING, "\n")<0){
    syslog(LOG_WARNING, "main: write_strings");
    return(-1);
  }

  /* Authenticate the user*/
#ifdef RETRY
  for(retry=RETRY;retry>0;retry--){
#else
  for(;;){
#endif
    /*Read the USER and PASS lines from the client */
    if((pw=pop3_in_get_pw(client_in, client_out))==NULL){
      syslog(LOG_WARNING, "main: pop3_in_get_pw");
      return(-1);
    }

    /*Read the server from the map*/
    if((server_port=gdbm_getserver(options, pw->pw_name))!=NULL){
      if((port=server_port_get_port(server_port))==NULL){
        port=DEFAULT_OUT_PORT;
      }
      servername=server_port_get_servername(server_port);
    }
    else{
      servername=NULL;
      port=NULL;
    }

    /*Log the session*/
    syslog(
      LOG_INFO, 
      "Connect: user=%s server=%s port=%s\n", 
      pw->pw_name,
      servername,
      port
    );


    /*Try again ig we didn't get anything useful*/
    if(server_port==NULL){
      sleep(PERDITION_ERR_SLEEP);
      if(write_strings(client_out, 2, ERR, " Could not determine server\n")<0){
        syslog(LOG_WARNING, "main: write_strings");
        return(-1);
      }
      continue;
    }

#ifdef AUTHENTICATE_IN
    /*Aluthenticate user locally - Not really necessary*/
    if(pop3_in_authenticate(pw, client_out)<1){
      syslog(LOG_INFO, "Local authenticatin failure");
      server_port_destroy(server_port);
      free(pw->pw_name);
      free(pw->pw_passwd);
      free(pw);
      continue;
    }
    
    /*Become someone else*/
    if(geteuid()){
      syslog(LOG_INFO, "Warning: not invoked as root, not setting uid and gid");
    }
    else if(daemon_setid(options.username, options.group)){
      syslog(LOG_WARNING, "Error setting group and userid, Exiting");
      exit(-1);
    }
#endif  


    /* Talk to the real pop server for the client*/
    if((server=tcp_socket_client_open(servername, port))<0){
      syslog(
        LOG_INFO, 
        "Could not connect to server, either server is "
        "unavailable or there is an invalid entry in %s for user %s",
        options.mapname,
        pw->pw_name
      );
      sleep(PERDITION_ERR_SLEEP);
      if(write_strings(client_out, 2, ERR, " Could not connect to server\n")<0){
        syslog(LOG_WARNING, "main: write_strings");
        return(-1);
      }
      continue;
    }

    /*We don't need this anymore*/
    server_port_destroy(server_port);
  
    /*Authenticate the user with the pop server*/
    if(pop3_out_authenticate(server, server, client_out, pw)<0){
      syslog(LOG_INFO, "Fail reauthentication for user %s\n", pw->pw_name);
      free(pw->pw_name);
      free(pw->pw_passwd);
      free(pw);
      continue;
    }

    /*If we get this far we are in heaven*/
    if(write_strings(client_out, 2, OK, " You're so in\n")<0){
      syslog(LOG_WARNING, "main: write_strings");
      return(-1);
    }

    break;
  }

  /*We need a buffer for reads and writes to the server*/
  if((buffer=(unsigned char *)malloc(BUFFER_SIZE*sizeof(unsigned char)))==NULL){
    syslog(LOG_WARNING, "main: malloc: %d, Exiting", strerror(errno));
    return(-1);
  }

  /*Let the client talk to the real server*/
  tcp_socket_pipe(server, server ,client_in ,client_out ,buffer ,BUFFER_SIZE);

  /*Time to die*/
  syslog(LOG_INFO, "Closing session for user %s\n", pw->pw_name);
  return(0);
}


/**********************************************************************
 * exit_cleanly
 * If we get a sinal then close everthing, log it and quit
 **********************************************************************/

void exit_cleanly(int i){
  syslog(LOG_INFO, "Exiting on signal\n");
  daemon_close_fd();
  exit(0);
}
