/**********************************************************************
 * pop3_in.c                                                 March 1999
 * Horms                                            horms@vergenet.net.
 * Verge Systems International                    http://vergenet.net./
 *
 * Handle pop commands from a client
 **********************************************************************/

#include "pop3_in.h"


/**********************************************************************
 * pop3_in_authenticate
 * Authenticate an incoming pop session
 * Not really needed if we are going to authenticate with an upstream
 * pop server but it may be useful in some cases
 *
 * Return 1 on success, 0 on failure and -1 on error.
 **********************************************************************/

#ifdef AUTHENTICATE_IN
int pop3_in_authenticate(struct passwd *pw, int err_fd){
  pam_handle_t *pamh=NULL;

  if((
     pam_retval=pam_start(SERVICE_NAME, pw->pw_name, &conv_struct, &pamh)
  )!=PAM_SUCCESS){
    syslog(LOG_WARNING, "main: pam_start: %s", strerror(errno));
    do_pam_end(pamh, EX_PAM_ERROR);
    return(-1);
  }
  if(do_pam_authentication(pamh, pw->pw_name, pw->pw_passwd)<0){
    sleep(POP3_IN_AUTH_FAIL_SLEEP);
    if(write_strings(err_fd,2,ERR," Authentication failure\n")<0){
      syslog(LOG_WARNING, "main: write_strings: %s", strerror(errno));
      do_pam_end(pamh, EXIT_SUCCESS);
      return(-1);
    }
    do_pam_end(pamh, EXIT_SUCCESS);
    return(0);
  }

  do_pam_end(pamh, EXIT_SUCCESS);
  return(1);
}
#endif


/**********************************************************************
 * pop3_in_get_pw
 * read USER and PASS commands and return them in a struct passwd *
 * allocated by this function
 **********************************************************************/

struct passwd *pop3_in_get_pw(int in_fd, int out_fd){
  queue *q;
  struct passwd *pw;
  pop3_token *user;
  pop3_token *pass;
  pop3_token *quit;
  pop3_token *token;


  if((pw=(struct passwd *)malloc(sizeof(struct passwd)))==NULL){
    syslog(LOG_WARNING, "pop3_in_get_pw: malloc: %s", strerror(errno));
    return(NULL);
  }

  if((user=create_pop3_token(user))==NULL){
    syslog(LOG_WARNING, "pop3_in_get_pw: create_pop3_token");
    return(NULL);
  }

  assign_pop3_token(user, POP3_IN_USTRING "USER", 4, -1);

  if((pass=create_pop3_token(pass))==NULL){
    syslog(LOG_WARNING, "pop3_in_get_pw: create_pop3_token");
    return(NULL);
  }

  assign_pop3_token(pass, POP3_IN_USTRING "PASS", 4, -1);

  if((quit=create_pop3_token(quit))==NULL){
    syslog(LOG_WARNING, "pop3_in_get_pw: create_pop3_token");
    return(NULL);
  }

  assign_pop3_token(quit, POP3_IN_USTRING "QUIT", 4, -1);

  pw->pw_name=NULL;
  while(1){
    if((q=read_pop3_line(in_fd, q))==NULL){
      syslog(LOG_WARNING, "pop3_in_get_pw: read_pop3_line");
      return(NULL);
    }

    if((q=pop(q, (queue_t *)&token))==NULL){
      syslog(LOG_WARNING, "pop3_in_get_pw: pop");
      return(NULL);
    }

    if(pop3_token_cmp(user, token)==0){
      destroy_pop3_token(token);
      if(pw->pw_name!=NULL){
        sleep(POP3_IN_ERR_SLEEP);
        if(write_strings(out_fd, 2, ERR, " USER is already set\n")<0){
          syslog(LOG_WARNING, "pop3_in_get_pw: write_strings");
          return(NULL);
        }
        destroy_queue(q, DESTROY_POP3_TOKEN);
        continue;
      }
      if(queue_length(q)!=1){
        sleep(POP3_IN_ERR_SLEEP);
        if(write_strings(out_fd, 2, ERR, " Try USER <username>\n")<0){
          syslog(LOG_WARNING, "pop3_in_get_pw: write_strings");
          return(NULL);
        }
        destroy_queue(q, DESTROY_POP3_TOKEN);
        continue;
      }

      if((q=pop( q, (queue_t *)&token))==NULL){
        syslog(LOG_WARNING, "pop3_in_get_pw: pop");
        return(NULL);
      }
      if((pw->pw_name=pop3_token_to_string(token, pw->pw_name))==NULL){
        syslog(LOG_WARNING, "pop3_in_get_pw: pop3_token_to_string");
        return(NULL);
      }
      unassign_pop3_token(token);
      destroy_pop3_token(token);
      destroy_queue(q, DESTROY_POP3_TOKEN);
      
      if(write_strings(out_fd, 4, OK, " USER ", pw->pw_name, " set\n")<0){
        syslog(LOG_WARNING, "pop3_in_get_pw: write_strings");
        return(NULL);
      }
      continue;
    }
    else if(pop3_token_cmp(pass, token)==0){
      if(pw->pw_name==NULL){
        sleep(POP3_IN_ERR_SLEEP);
        if(write_strings(out_fd, 2, ERR, " USER not yet set\n")<0){
          syslog(LOG_WARNING, "pop3_in_get_pw: write_strings");
          return(NULL);
        }
        destroy_queue(q, DESTROY_POP3_TOKEN);
        continue;
      }
      if(queue_length(q)!=1){
        sleep(POP3_IN_ERR_SLEEP);
        if(write_strings(out_fd, 2, ERR, " Try PASS <password>\n")<0){
          syslog(LOG_WARNING, "pop3_in_get_pw: write_strings");
          return(NULL);
        }
        destroy_queue(q, DESTROY_POP3_TOKEN);
        continue;
      }
      if((q=pop( q, (queue_t *)&token))==NULL){
        syslog(LOG_WARNING, "pop3_in_get_pw: pop");
        return(NULL);
      }
      if((pw->pw_passwd=pop3_token_to_string(token, pw->pw_passwd))==NULL){
        syslog(LOG_WARNING, "pop3_in_get_pw: pop3_token_to_string");
        return(NULL);
      }
      unassign_pop3_token(token);
      destroy_pop3_token(token);
      destroy_queue(q, DESTROY_POP3_TOKEN);
      free(user);
      free(pass);
      free(quit);
      return(pw);
    }
    else if(pop3_token_cmp(quit, token)==0){
      syslog(LOG_INFO, "Closing NULL session: username=%s", pw->pw_name);
      if(write_strings(out_fd, 2, OK, " QUIT\n")<0){
        syslog(LOG_WARNING, "pop3_in_get_pw: write_strings");
        return(NULL);
      }
      exit(0);
    }
    else{
        sleep(POP3_IN_ERR_SLEEP);
      if(write_strings(
        out_fd, 2, ERR, " Command must be one of USER, PASS or QUIT\n"
      )<0){
        syslog(LOG_WARNING, "pop3_in_get_pw: write_strings");
        return(NULL);
        destroy_pop3_token(token);
        destroy_queue(q, DESTROY_POP3_TOKEN);
      }
    }
  }

  /*Here so compilers won't barf*/
  return(NULL);
}
