/**********************************************************************
 * tcp_socket_client.c                                       April 1999
 * Horms                                            horms@verge.ing.to.
 * Verge Systems International                http://www.verge.ing.to./
 *
 **********************************************************************/

#include "tcp_socket_client.h"


/**********************************************************************
 * tcp_socket_client_open
 * Open a socket connection as a client
 **********************************************************************/

int tcp_socket_client_open(char *host, char *port){
  int out;
  struct sockaddr_in to;
  struct hostent *hp;

  /*Get details of the forign 'to' host*/
  if((hp=gethostbyname(host)) == NULL){
    syslog(LOG_WARNING, "tcp_socket_client_open: unknown host %s\n", host);
    return(-1);
  }

  /*Create socket for 'to' connection*/
  if((out=socket(AF_INET, SOCK_STREAM, 0)) < 0){
    syslog(
      LOG_WARNING, 
      "tcp_socket_client_open: socket",
      strerror(errno)
    );
    return(-1);
  }

  /*Fill in port information for 'to'*/
  bzero((struct sockaddr *)&to, sizeof(to));
  bcopy(hp->h_addr, (struct sockaddr *)&to.sin_addr, hp->h_length);
  to.sin_family = hp->h_addrtype;
  if((to.sin_port=tcp_socket_client_portno(port))==0){
    syslog(LOG_WARNING, "tcp_socket_client_open: tcp_socket_client_portno");
    return(-1);
  }

  /*Connect to foreign 'to' server */
  if(connect(out, (struct sockaddr *)&to, sizeof(to))<0){
    syslog(
      LOG_WARNING, 
      "tcp_socket_client_open: connect",
      strerror(errno)
    );
    return(-1);
  }

  return(out);
}


/**********************************************************************
 * tcp_socket_client_portno
 * Return the port number of a service given as a string either
 * the port number or the service name as per /etc/services
 **********************************************************************/

unsigned short int tcp_socket_client_portno(const char *port){
  struct servent *ent;
  int offset;
  unsigned short int portno;

  for(offset=strlen(port)-1;offset>-1;offset--){
    if(!isdigit(*(port+offset))){
      break;
    }
  }

  if(str_is_digit(port)){
    portno=htons(atoi(port));
  }
  else{
    if((ent=getservbyname(port, "tcp"))==NULL){
      syslog(
        LOG_WARNING, 
	"tcp_socket_client_portno: getservbyname: %s",
	strerror(errno)
      );
      return(0);
    }
    portno=(unsigned short int)ent->s_port;
  }
  return(portno);
}
  
   
